# Redis-MQ

A lightweight message queue system built on Redis, supporting cron jobs and one-off tasks.

## Installation

```bash
npm install @leotermine/redismq
```

## Quick Start

```typescript
import { QueueManager } from "@leotermine/redismq";
import { Redis } from "ioredis";

// 1. Configure Redis
const redisConfig = {
  port: 6379,
  host: 'localhost', // or the host in use
  username: '',
  password: '',
  maxRetriesPerRequest: null,
  enableReadyCheck: false,
};

// 2. Initialize Redis client
const client = new Redis(redisConfig);

// 3. Initialize Queue Manager
const contextApp = {}; // Optional context passed to job handlers
const app = QueueManager.init(client, contextApp, 1); // Last param is concurrency

// 4. Define a job
const myJob = {
  path: 'category/job-name', // Must follow 'category/name' format
  run: (ctx, job) => {
    console.log('Job running!', job);
  },
  options: {
    repeat: {
      pattern: '* * * * *' // Cron pattern (optional)
    },
    attempts: 3 // Retry attempts (optional)
  }
};

// 5. Register job
app.registerJob(myJob);

// 6. Start processing
app.processJobs();

// 7. Add one-off job (optional)
app.addJob('category/job-name', {
  data: 'your-data'
}, {
  attempts: 3
});
```

## Job Types

### Cron Jobs
```typescript
const cronJob = {
  path: 'crons/hello-world',
  run: (ctx, job) => {
    console.log("Running on schedule");
  },
  options: {
    repeat: {
      pattern: "* * * * *" // Runs every minute
    }
  }
};
```

### One-off Jobs
```typescript
const oneOffJob = {
  path: 'tasks/send-email',
  run: (ctx, job) => {
    console.log("Running once");
  }
};
```

## API Reference

### QueueManager
- `init(redisClient, context, concurrency)`: Initialize queue manager
- `registerJob(job)`: Register a new job type
- `addJob(path, data, options)`: Add a job instance
- `processJobs()`: Start processing jobs
- `getAllJobs()`: Get all jobs in queues

### Job Options
```typescript
interface JobOptions {
  repeat?: {
    pattern: string;    // Cron pattern
  };
  attempts?: number;    // Retry attempts
  _id?: string;        // Custom job ID
  priority?: number;    // Job priority
  delayUntil?: Date;   // Delay execution
}
```

## License

MIT
